//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>

int main()
{
  auto fib = [](this auto self, int n) {
               if (n <= 1) return 1;  // we start with 1 and 1
               return self(n - 1) + self(n - 2);
             };

  for (int n : {8, 12, 16}) {
    std::println("the {:2}-th Fibonacci number is {:4}", n, fib(n));
  }
}

