//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <vector>
#include <ranges>

int main()
{
  std::vector<int> coll{1, 2, 3, 4, 5, 6, 7, 8};
  std::println("{}", coll);

  // modify first five values:
  for (auto& elem : coll | std::views::take(5)) {
    elem = elem * elem;    // OK
  }
  std::println("{}", coll);

  // cannot modify first five values:
  for (auto& elem : coll | std::views::take(5) | std::views::as_const) {
    elem = elem * elem;    // ERROR: elem is const
  }
  std::println("{}", coll);
}

