//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <vector>
#include <mdspan>
#include "print2d.hpp"

int main()
{
  std::vector vec{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12};

  // 4 rows of 5 times every second value (overlapping):
  std::mdspan mds{vec.data(), 
                  std::layout_stride::mapping{std::extents{4, 5},
                                              std::array{1, 2}}};
  print2D(mds);

  // append new elements to the collection:
  vec.insert(vec.end(), {101, 102, 103, 104, 105, 106, 107, 108, 109});

  // reinitialize the mdspan with rows of 7 times every third value:
  mds = std::mdspan{vec.data(),
                    std::layout_stride::mapping{std::extents{vec.size()-18, 7},
                                                std::array{1, 3}}};
  print2D(mds); 
}

