//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <string>
#include <vector>
#include <array>
#include <ranges>

int main()
{
  std::vector<std::string> rg1{"tic", "tac", "toe"};
  std::vector<std::string> rg2{"none"};
  std::vector<std::string> rg3{"one", "two"};
  std::array coll{rg1, rg2, rg3};

  std::println("coll:  {}", coll);                                // ranges of ranges
  std::println("join:  {}", coll | std::views::join);             // joined
  std::println(" +sep: {}", coll | std::views::join_with("--"));  // with separator
  std::println("       {}", std::views::join_with(coll, "--"));   // with separator
}

