//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <vector>
#include <ranges>

int main()
{
  std::vector<std::string> coll{"tic", "tac", "toe", "42", "end"};
  std::println("{}", coll);

  for (const auto& sub : coll | std::views::slide(3)) {
    std::println("- process {}", sub);
    sub.front() += '*';   // OK compiles and modifies elements of coll
  }

  std::println("{}\n", coll);
}

