//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <vector>
#include <ranges>

auto getColl()
{
  return std::vector{10, 20, 30};
}

int main()
{
  auto coll = getColl();
  std::println("coll:  {}", coll);                 // OK
  auto collFront = getColl().front();
  std::println("       front: {}", collFront);     // OK

  auto ecoll = std::views::enumerate(getColl());
  std::println("ecoll: {}", ecoll);                // OK
  auto ecollFront = std::views::enumerate(getColl()).front();
  std::println("       front: {}", ecollFront);    // Undefined Behavior
}

