//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <iostream>
#include <print>
#include "fibo3.hpp"

int main()
{
  // sequence of 20 Fibonacci numbers:
  for (auto val : fibonacci(20)) {
    std::cout << val << ' ';
  }
  std::cout << '\n';

  // sequence of 15 Fibonacci numbers:
  std::println("{}", fibonacci(15));
}

