//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <iostream>
#include "cororectrace.hpp"

int main()
{
  std::cout << "All files:\n";
  for (const auto& f : allFiles(".")) {
    std::cout << "  " << f << "\n";
  }
}

