//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <iostream>
#include <ranges>

int main()
{
  for (const auto& elem : std::views::repeat("hello", 4)) {
    std::cout << elem << ' ';  // prints: hello hello hello hello
  }
  std::cout << '\n';

  for (int i : std::views::repeat(42)) {
    std::cout << i << ' ';     // prints: 42 42 42 ...  (unlimited)
  }
  std::cout << '\n';
}

