//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <ranges>
#include "tracedcoll.hpp"

auto getColl()
{
  return TracedColl<std::string>{"One", "Two", "Three"};
}

int main()
{
  // move elements of returned temporary collection into a view
  // that refers to the second to fourth element:
  auto v = getColl() | std::views::drop(1) | std::views::take(3);

  println("\nv: {}\n", v);

  // elements of the moved temporary collection will be destroyed here
}

