//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <vector>
#include <mdspan>
#include "print2d.hpp"

int main()
{
  std::vector coll{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12};

  // mdspan of rows of 3 values:
  std::mdspan mds{coll.data(), coll.size() / 3, 3};
  print2D(mds);

  // mdspan of rows of 3 values with column-oriented layout:
  std::mdspan<int, std::dextents<size_t, 2>,
              std::layout_left> mdsl{coll.data(),coll.size() / 3, 3};  
  print2D(mdsl);
}

