//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <iostream>
#include <string>

class Person {
 private:
  std::string name;
 public:
  Person (const std::string& n) 
   : name{n} {
  }
  //...
  friend std::ostream& operator<< (std::ostream& os, const Person& p) {
    return os << '[' << p.name << ']';
  }

  void print () const {
    std::cout << *this << '\n';  // delegate work to the output operator
  }

  void clear() {                 // clear the name
    name = "";
  }
};

