//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <iostream>
#include "array2d.hpp"

int main()
{
  Array2D<std::string, 2, 3> arr{"Tic", "Tac", "Toe"};
  arr.print();

  std::cout << @\TB{arr[0, 1]}@ << '\n';

  @\TB{arr[1, 2]}@ = "End";
  arr.print();
}

