//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <string>
#include <vector>
#include <ranges>

int main()
{
  std::vector<std::string> coll = {"tic", "tac", "toe", "end"};
  std::println("coll:     {}\n", coll);

  auto first3cp = std::ranges::to<std::vector>(coll | std::views::take(3));
  std::println("coll:     {}", coll);
  std::println("first3cp: {}\n", first3cp);

  auto first3mv = std::ranges::to<std::vector>(coll | std::views::take(3)
                                                    | std::views::as_rvalue);
  std::println("coll:     {}", coll);
  std::println("first3mv: {}\n", first3mv);
}

