//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <string>
#include <vector>
#include <functional>
#include <cmath>
#include <ranges>

int main()
{
  std::vector coll{47, 11, 0, 8, 16, 10, 11};
  std::println("{}\n", coll);

  for (auto rg : std::views::chunk_by(coll, std::less{})) {
    std::println("{} ", rg);
  }
  std::println("");

  auto allEvenOrOdd = [] (auto x, auto y) {
    return std::abs(x % 2) == std::abs(y % 2);
  };
  for (auto rg : coll | std::views::chunk_by(allEvenOrOdd)) {
    std::println("{} ", rg);
  }
  std::println("");
}

