//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <string>
#include <vector>
#include <ranges>

int main()
{
  std::vector<std::string> coll{"tic", "tac", "toe", "47", "11"};
  std::println("coll: {}\n", coll); 

  auto vec = coll | std::views::adjacent<2>
                  | std::views::as_rvalue               // BAD
                  | std::ranges::to<std::vector>();
  std::println("coll: {}", coll);
  std::println("vec:  {}\n", vec);
}

