//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <vector>
#include <ranges>

int main()
{
  std::vector coll{1, 4, 7, 10};

  // define view for even elements of coll:
  auto even = [](int i){ return i%2 == 0;};
  auto collEven = coll | std::views::filter(even);

  // add just some debug output:
  std::println("even: {}", collEven);

  // modify underlying range:
  coll[0] = 0;

  // print collection and view:
  std::println("coll: {}", coll);
  std::println("even: {}", collEven);
}

