//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <string>
#include <vector>
#include <ranges>

int main()
{
  std::vector<std::string> coll{"tic", "tac", "toe", "42", "end"};
  std::println("{}\n", coll);

  for (auto [a, b, c] : std::views::adjacent<3>(coll)) {
    std::println("{}:{}:{}", a, b, c);
  }
  std::println("");

  for (auto [a, b, c, d] : coll | std::views::adjacent<4>) {
    std::println("{}:{}:{}:{}", a, b, c, d);
  }
}

