//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <vector>
#include <mdspan>
#include "print2d.hpp"

int main()
{
  std::vector vec{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12};

  // mdspan with rows of 3 values:
  std::mdspan mds{vec.data(), vec.size() / 3, 3};
  print2D(mds);

  // add more elements:
  // - reallocation invalidates the mdspan
  vec.push_back(42);

  // do NOT use the mdspan here without reinitialization

  // reinitialize mdspan for new number of elements:
  mds = std::mdspan{vec.data(), vec.size() / 3, 3};
  print2D(mds);
}

