//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <iostream>
#include <format>
#include <print>
#include <vector>
#include <ranges>

int main()
{
  std::vector<int> coll{0, 8, 15, 47, 11, -1, 42};

  // print whole collection:
  std::cout << std::format("coll: {}\n", coll);   // output range with format()
  std::print("coll: {}\n", coll);                 // output range with print()
  std::println("coll: {}", coll);                 // output range with println()

  // print view with every second element:
  std::println("every 2nd: {}", coll | std::views::stride(2));  // output view
}

