//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <string>
#include <vector>
#include <map>

class Data {
  std::map<std::string, std::vector<std::string>> data;
 public:
  Data() = default;

  void insert(const std::string& key, const std::vector<std::string>& val) {
    data[key] = val;
  }
   
  char operator[] (const std::string& key, std::size_t x, std::size_t y) const {
    return data.at(key).at(x).at(y);
  }
};

