//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <vector>
#include <mdspan>
#include "print2d.hpp"

int main()
{
  std::vector coll = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12};

  // define mdspan for dynamic number of rows of 3 values:
  std::mdspan<int, std::extents<size_t,
                                std::dynamic_extent,
                                3>
             > mds{coll.data(), coll.size()/3};
  print2D(mds);

  // append new elements to the collection:
  coll.insert(coll.end(), {101, 102, 103, 104, 105, 106, 107, 108, 109});

  // reinitialize the mdspan with the new number of rows:
  mds = decltype(mds){coll.data(), coll.size()/3};
  print2D(mds); 
}

