//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <vector>
#include <ranges>

int main()
{
  std::vector coll{1, 2, 3, 4, 5, 6, 7, 8, 9};

  std::println("{}", coll);                               // 1 2 3 4 5 6 7 8 9
  std::println("{}", std::ranges::stride_view{coll, 3});  // 1 4 7
  std::println("{}", coll | std::views::stride(3));       // 1 4 7
}

