//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <string>
#include <vector>
#include <ranges>

std::vector<std::string> getColl()
{
  return {"someprettylongstring", "tic", "tac", "toe"};
}

int main()
{
  std::println("{}\n", getColl()); 

  auto vec = getColl() | std::views::slide(2)
                       | std::ranges::to<std::vector>();  // FATAL ERROR
  std::println("{}\n", vec);
}

