//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


template<typename CollT>
void replacePrefixesWith(CollT& coll, const std::string& prefix,
                                      const std::string& replacement)
{
  for (auto& elem : coll) {
    if (elem.starts_with(prefix)) {
      elem.replace(0, prefix.size(), replacement);
    }
  }
}

