//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <utility>
#include <generator>

std::generator<long> fibonacci(int num) 
{
  // first two numbers are 1:               
  long a = 1;
  long b = 1;
  co_yield a;        // SUSPEND with first number
  co_yield b;        // SUSPEND with second number

  // loop over the remaining num-2 sums of previous two numbers:
  for (int i = 2; i < num; ++i) {
    auto sum = a + b;
    co_yield sum;    // SUSPEND with sum of previous two numbers
    a = std::exchange(b, sum);
  }
}

