//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


import std;                       // make the whole C++ standard library available

int main()
{
  std::map<std::string, std::chrono::system_clock::time_point> dates;

  dates["now"] = std::chrono::system_clock::now();

  for (const auto& [key, val] : dates | std::views::take(5)) {
    std::cout << key << ": " << val << '\n';
  }
}

