//********************************************************
// The following code example is taken from the book
//  C++23 - The Complete Guide
//  by Nicolai M. Josuttis (www.josuttis.com)
//  http://www.cppstd23.com
//
// The code is licensed under a
//  Creative Commons Attribution 4.0 International License
//  http://creativecommons.org/licenses/by/4.0/
//********************************************************


#include <print>
#include <string>
#include <vector>
#include <ranges>

int main()
{
  std::vector<std::string> coll{"tic", "tac", "toe", "42", "end"};
  std::println("{}", coll);

  // split into sub-views of 2 elements (last may have less):
  const auto cv = coll | std::views::chunk(2);  // const not propagated

  for (const auto& sub : cv) {                  // const not propagated
    // modify first element of each subview:
    std::println("- process {}", sub);
    sub.front() += '*';           // OOPS: \IBcompiles and modifies elements of coll
  }
  std::println("{}\n", coll);
}

